author = "Marc Ruef"
license = "(c) 2010 by Marc Ruef"
version = "1.0"
categories = {"default", "safe", "scip"}

require("scipreporting")
require("shortport")

description = [[ This script identifies web servers which are still using default install pages ]]

portrule = shortport.port_or_service({80, 81, 8080, 443, 8443}, {"http", "https"}, {"tcp"})

action = function(host, port)
	HttpAnswer = http.get(host, port, "/")

	if HttpAnswer.body ~= nil and HttpAnswer.body ~= "" then
		local mt = {
			{pattern="Welcome To Your WebSTAR Home Page", product="WebSTAR Server"},
			{pattern="Test Page for Apache Installation on Web Site", product="Apache httpd"},
			{pattern="If you're seeing this page via a web browser, it means you've setup Tomcat successfully.", product="Apache Tomcat"},
			{pattern="The site you were trying to reach does not currently have a default page. It may be in the process of being upgraded and configured.", product="Microsoft IIS"},
			{pattern="Dies ist die Standardhomepage, die mit dem ersten Ausfhren von Setup zu IIS&nbsp;4.0 auf Ihrem Computer installiert wird.", product="Microsoft IIS 4.0 (Option Pack, Windows German Edition)"},
			{pattern="Microsoft Windows NT 4.0 Option Pack provides enhanced Web", product="Microsoft IIS 4.0 (Option Pack, Windows English Edition)"},
			{pattern="/help/help6_client.nsf", product="IBM Lotus Domino 6.0"},
			{pattern="iPlanet Web Server, Enterprise Edition 6.0", product="Sun iPlanet Web Server 6.0"},
			{pattern="By default, the username is 'admin' and the password is blank.", product="Sambar Server"},
			{pattern="Welcome to NetWare 6", product="Novell NetWare 6"},
			{pattern="Default BEA WebLogic Server Web Server Index Page", product="BEA WebLogic Server 8.x"},
			{pattern="Read Me - Welcome to BEA WebLogic Server", product="BEA WebLogic Server"},
			{pattern="Your server is now running and you can start deploying your J2EE", product="Oracle Application Server Containers for J2EE"},
			{pattern="Welcome to Windows 2000 Internet Services", product="Microsoft IIS 5.0 (Windows 2000)"}
			}	

		local result = ""
		for i=1, #mt, 1 do
			if string.match(HttpAnswer.body, mt[i].pattern) then
				resultdata = "Pattern:\t" .. mt[i].pattern .. "\nProduct:\t" .. mt[i].product
				if result == nil or result == "" then
					result = resultdata
				else
					result = result .. "\n\n" .. resultdata
				end
			end
	        end
	
		if result ~= "" then
			sAccuracy	= SCIP_DEFAULT_APPFINGERPRINT_ACCURACY
			sTesttype	= "Exploiting"
			sTestsource	= "Application Fingerprinting"
			sOutput		= result
			sDescription	= "The web server is still using the install page which is established during the initial installation of the software. An attacker might use this information to approach target-oriented attacks."
			
			return scipreport(130, sAccuracy, sTesttype, sTestsource, version, sOutput, sDescription)
		end	
	end
end
