author = "Marc Ruef"
license = "(c) 2010 by scip AG"
version = "1.0-hd10"
categories = {"default", "safe", "scip"}

require("scipreporting_hashdays")
require("shortport")

description = [[ This script is identifying web servers which support the options method ]]

portrule = shortport.port_or_service({80, 81, 8080, 443, 8443}, {"http", "https"}, {"tcp"})

action = function(host, port)
	local socket = nmap.new_socket()
	local result
	local status = true
	local serverline
	local optionsupported = false

	local err_catch = function()
		socket:close()
	end

	local try = nmap.new_try(err_catch)

	socket:set_timeout(5000)
	try(socket:connect(host.ip, port.number, port.protocol))
	try(socket:send("OPTIONS * HTTP/1.0\n\n"))
        while status do
		status, result = socket:receive_lines(10);

		serverline = string.match(result, "(Allow:(.-))[\r\n]")
		if serverline ~= nil and serverline ~= "" then
			sOutput = serverline
			optionsupported = true
			break
		else
			serverline = string.match(result, "(Public:(.-))[\r\n]")
			if serverline ~= nil and serverline ~= "" then
				sOutput = serverline
				optionsupported = true
				break
			end
		end
	end
	socket:close()

	if optionsupported == true then
		sAccuracy	= SCIP_DEFAULT_EXPLOITING_ACCURACY
		sTesttype	= "Exploiting"
		sTestsource	= "HTTP-Response (Allow Line)"
		sDescription	= "The web server is supporting the OPTIONS method which announces the supported http methods of the web server. An attacker might use this information for http fingerprinting or target-oriented attacks."

		return scipreport(7, sAccuracy, sTesttype, sTestsource, version, sOutput, sDescription)
	end
end
