author = "Marc Ruef"
license = "(c) 2010 by scip AG"
version = "1.0-hd10"
categories = {"default", "safe", "scip"}

description = [[ This script is enumerating robots.txt files on web servers ]]

require("scipreporting_hashdays")
require("shortport")
require("strbuf")
require("http")

portrule = shortport.port_or_service({80, 81, 8080, 443, 8443}, {"http", "https"}, {"tcp"})
local last_len = 0

-- split the output in 50 character length lines
local function buildOutput(output, w)
        local nl

	if w:len() == 0 then
		return nil
	end

	-- check for duplicates
	for i,v in ipairs(output) do
		if w == v or w == v:sub(2, v:len()) then
			return nil
		end
	end

	-- format lines
	if last_len == 0 or last_len + w:len() <= 50 then
		last_len = last_len + w:len()
		nl = ''
	else
		last_len = 0
		nl = '\n'
	end

	output = output .. (nl .. w)
end

-- parse all disallowed entries in body and add them to a strbuf
local function parse_robots(body, output)
	for line in body:gmatch("[^\r\n]+") do
		for w in line:gmatch('[Dd]isallow:%s*(.*)') do
			w = w:gsub("%s*#.*", "")
			buildOutput(output, w)
		end
	end

	return #output
end

action = function(host, port)
        local dis_count, noun
	local answer = http.get(host, port, "/robots.txt" )

	if answer.status ~= 200 then
		return nil
	end

	local v_level = nmap.verbosity() + (nmap.debugging()*2)
	local output = strbuf.new()
	local detail = 15

	dis_count = parse_robots(answer.body, output)

	if dis_count == 0 then
		return
	end

	-- check we have enough entries
	if detail > dis_count then
		detail = dis_count
	end

	noun = dis_count == 1 and "entry " or "entries "

	local shown = (detail == 0 or detail == dis_count)
                 and "\n" or '(' .. detail .. ' shown)\n'

	sAccuracy	= SCIP_DEFAULT_EXPLOITING_ACCURACY
	sTesttype	= "Exploiting"
	sTestsource	= "Exploiting"		
	sOutput		= (dis_count .. " disallowed " .. noun .. shown .. table.concat(output, ' ', 1, detail) .. ":\n\n" .. answer.body)
	sDescription	= "The web server is providing a robots.txt file which indicates information for search engines. An attacker might enumerate the provided definitions to identify hidden or suspicious sources."

	return scipreport(3, sAccuracy, sTesttype, sTestsource, version, sOutput, sDescription)
end
